# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from functools import cached_property
from typing import TypeVar

from gvm.errors import GvmError
from gvm.xml import Element, parse_xml

from ._request import Request


class StatusError(GvmError):
    """
    The response had an error status

    May be raised when calling `response.raise_for_status()`
    """

    def __init__(self, message: str | None, *args, response: "Response"):
        super().__init__(message, *args)
        self.response = response
        self.request = response.request


Self = TypeVar("Self", bound="Response")


class Response:
    """
    A GMP Response
    """

    def __init__(self, *, request: Request, data: bytes) -> None:
        """
        Create a Response object

        Args:
            request: The request corresponding to this response
            data: The data of the response
        """
        self._request = request
        self._data = data
        self.__xml: Element | None = None

    def __root_element(self) -> Element:
        if self.__xml is None:
            self.__xml = self.xml()
        return self.__xml

    def xml(self) -> Element:
        """
        Return the response data as XML Element

        Raises XmlError if the data is not valid XML.
        """
        return parse_xml(self.data)

    @property
    def data(self) -> bytes:
        """
        Return the data of the response as bytes
        """
        return self._data

    @property
    def request(self) -> Request:
        """
        Return the corresponding request of this response
        """
        return self._request

    @cached_property
    def status_code(self) -> int | None:
        """
        The status code of the response

        Returns:
            The status code or None if the response data doesn't contain a valid
            status code.
        """
        root = self.__root_element()
        try:
            status = root.attrib["status"]
            return int(status)
        except (KeyError, ValueError):
            return None

    @property
    def is_success(self) -> bool:
        """
        Returns True if the response contains a success status code
        """
        status = self.status_code
        return status is not None and 200 <= status <= 299

    def raise_for_status(self: Self) -> Self:
        if self.is_success:
            return self
        raise StatusError(
            f"Invalid status code {self.status_code}", response=self
        )

    def __bytes__(self) -> bytes:
        """
        Return the data as bytes
        """
        return self._data

    def __str__(self) -> str:
        """
        Return the data as string
        """
        return self._data.decode()



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v227 import AliveTest


class GetAliveTestFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            AliveTest.from_string("foo")

    def test_none_or_empty(self):
        ct = AliveTest.from_string(None)
        self.assertIsNone(ct)
        ct = AliveTest.from_string("")
        self.assertIsNone(ct)

    def test_scan_config_default(self):
        ct = AliveTest.from_string("Scan Config Default")
        self.assertEqual(ct, AliveTest.SCAN_CONFIG_DEFAULT)

    def test_icmp_ping(self):
        ct = AliveTest.from_string("ICMP Ping")
        self.assertEqual(ct, AliveTest.ICMP_PING)

    def test_tcp_ack_service_ping(self):
        ct = AliveTest.from_string("TCP-ACK Service Ping")
        self.assertEqual(ct, AliveTest.TCP_ACK_SERVICE_PING)

    def test_tcp_sync_service_ping(self):
        ct = AliveTest.from_string("TCP-SYN Service Ping")
        self.assertEqual(ct, AliveTest.TCP_SYN_SERVICE_PING)

    def test_arp_ping(self):
        ct = AliveTest.from_string("ARP Ping")
        self.assertEqual(ct, AliveTest.ARP_PING)

    def test_icmp_and_tcp_ack_service_ping(self):
        ct = AliveTest.from_string("ICMP & TCP-ACK Service Ping")
        self.assertEqual(ct, AliveTest.ICMP_AND_TCP_ACK_SERVICE_PING)

    def test_icmp_and_arp_ping(self):
        ct = AliveTest.from_string("ICMP & ARP Ping")
        self.assertEqual(ct, AliveTest.ICMP_AND_ARP_PING)

    def test_tcp_ack_service_and_arp_ping(self):
        ct = AliveTest.from_string("TCP-ACK Service & ARP Ping")
        self.assertEqual(ct, AliveTest.TCP_ACK_SERVICE_AND_ARP_PING)

    def test_icmp_tcp_ack_service_and_arp_ping(self):
        ct = AliveTest.from_string("ICMP, TCP-ACK Service & ARP Ping")
        self.assertEqual(ct, AliveTest.ICMP_TCP_ACK_SERVICE_AND_ARP_PING)

    def test_consider_alive(self):
        ct = AliveTest.from_string("Consider Alive")
        self.assertEqual(ct, AliveTest.CONSIDER_ALIVE)



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .system_reports import GmpGetSystemReportsTestMixin


class Gmpv224GetSystemReportsTestCase(
    GmpGetSystemReportsTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .tickets import (
    GmpCloneTicketTestMixin,
    GmpCreateTicketTestMixin,
    GmpDeleteTicketTestMixin,
    GmpGetTicketsTestMixin,
    GmpGetTicketTestMixin,
    GmpModifyTicketTestMixin,
)


class Gmpv224DeleteTicketTestCase(GmpDeleteTicketTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTicketTestCase(GmpGetTicketTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTicketsTestCase(GmpGetTicketsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneTicketTestCase(GmpCloneTicketTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateTicketTestCase(GmpCreateTicketTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyTicketTestCase(GmpModifyTicketTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetVulnerabilitiesTestMixin:
    def test_get_vulnerabilities(self):
        self.gmp.get_vulnerabilities()

        self.connection.send.has_been_called_with(b"<get_vulns/>")

    def test_get_vulnerabilities_with_filter_string(self):
        self.gmp.get_vulnerabilities(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_vulns filter="foo=bar"/>'
        )

    def test_get_vulnerabilities_with_filter_id(self):
        self.gmp.get_vulnerabilities(filter_id="f1")

        self.connection.send.has_been_called_with(b'<get_vulns filt_id="f1"/>')



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyScanConfigSetNameTestMixin:
    def test_modify_scan_config_set_name(self):
        self.gmp.modify_scan_config_set_name("c1", "foo")

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1"><name>foo</name></modify_config>'
        )

    def test_modify_scan_config_set_name_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_name(config_id=None, name="name")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_name("", name="name")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_name(config_id="", name="name")

    def test_modify_scan_config_set_name_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_name(config_id="c", name="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_name(config_id="c", name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_name("c", "")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


import httpx


class OpenvasdAPI:
    def __init__(
        self, client: httpx.Client, *, suppress_exceptions: bool = False
    ):
        """
        Initialize the OpenvasdAPI entry point.

        Args:
            client: An initialized `httpx.Client` configured for communicating
                    with the openvasd server.
            suppress_exceptions: If True, suppress exceptions and return structured error responses.
                Default is False, which means exceptions will be raised.
        """
        self._client = client
        self._suppress_exceptions = suppress_exceptions



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetScannersTestMixin:
    def test_get_scanners(self):
        self.gmp.get_scanners()

        self.connection.send.has_been_called_with(b"<get_scanners/>")

    def test_get_scanners_with_filter_string(self):
        self.gmp.get_scanners(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_scanners filter="foo=bar"/>'
        )

    def test_get_scanners_with_filter_id(self):
        self.gmp.get_scanners(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_scanners filt_id="f1"/>'
        )

    def test_get_scanners_with_trash(self):
        self.gmp.get_scanners(trash=True)

        self.connection.send.has_been_called_with(b'<get_scanners trash="1"/>')

        self.gmp.get_scanners(trash=False)

        self.connection.send.has_been_called_with(b'<get_scanners trash="0"/>')

    def test_get_scanners_with_details(self):
        self.gmp.get_scanners(details=True)

        self.connection.send.has_been_called_with(
            b'<get_scanners details="1"/>'
        )

        self.gmp.get_scanners(details=False)

        self.connection.send.has_been_called_with(
            b'<get_scanners details="0"/>'
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GMPCloneReportConfigTestMixin:
    def test_clone_report_config(self):
        self.gmp.clone_report_config("report_config_id")

        self.connection.send.has_been_called_with(
            b"<create_report_config>"
            b"<copy>report_config_id</copy>"
            b"</create_report_config>",
        )

    def test_clone_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.clone_report_config(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.clone_report_config("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.permissions import (
    GmpClonePermissionTestMixin,
    GmpCreatePermissionTestMixin,
    GmpDeletePermissionTestMixin,
    GmpGetPermissionsTestMixin,
    GmpGetPermissionTestMixin,
    GmpModifyPermissionTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeletePermissionTestCase(
    GmpDeletePermissionTestMixin, GMPTestCase
):
    pass


class Gmpv225GetPermissionTestCase(GmpGetPermissionTestMixin, GMPTestCase):
    pass


class Gmpv225GetPermissionsTestCase(GmpGetPermissionsTestMixin, GMPTestCase):
    pass


class Gmpv225ClonePermissionTestCase(GmpClonePermissionTestMixin, GMPTestCase):
    pass


class Gmpv225CreatePermissionTestCase(
    GmpCreatePermissionTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyPermissionTestCase(
    GmpModifyPermissionTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from numbers import Real

from gvm._enum import Enum

Severity = Real


class SeverityLevel(Enum):
    """Enum for severity levels"""

    HIGH = "High"
    MEDIUM = "Medium"
    LOW = "Low"
    LOG = "Log"
    ALARM = "Alarm"
    DEBUG = "Debug"



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import InvalidArgumentType, RequiredArgument


class GmpModifyAgentControllerScanConfigTestMixin:
    def test_modify_agent_control_scan_config_full(self):
        config = {
            "agent_defaults": {
                "agent_control": {
                    "retry": {
                        "attempts": 6,
                        "delay_in_seconds": 60,
                        "max_jitter_in_seconds": 10,
                    }
                },
                "agent_script_executor": {
                    "bulk_size": 1,
                    "bulk_throttle_time_in_ms": 100,
                    "indexer_dir_depth": 10,
                    "scheduler_cron_time": [
                        "0 23 * * *",
                        "0 22 * * *",
                    ],
                },
                "heartbeat": {
                    "interval_in_seconds": 600,
                    "miss_until_inactive": 1,
                },
            },
            "agent_control_defaults": {
                "update_to_latest": True,
            },
        }

        self.gmp.modify_agent_control_scan_config(
            "3b4be213-281f-49ee-b457-5a5f34f71510",
            config=config,
        )

        self.connection.send.has_been_called_with(
            b'<modify_agent_control_scan_config agent_control_id="3b4be213-281f-49ee-b457-5a5f34f71510">'
            b"<config_defaults>"
            b"<agent_defaults>"
            b"<agent_control>"
            b"<retry>"
            b"<attempts>6</attempts>"
            b"<delay_in_seconds>60</delay_in_seconds>"
            b"<max_jitter_in_seconds>10</max_jitter_in_seconds>"
            b"</retry>"
            b"</agent_control>"
            b"<agent_script_executor>"
            b"<bulk_size>1</bulk_size>"
            b"<bulk_throttle_time_in_ms>100</bulk_throttle_time_in_ms>"
            b"<indexer_dir_depth>10</indexer_dir_depth>"
            b'<scheduler_cron_time is_list="1">'
            b"<item>0 23 * * *</item>"
            b"<item>0 22 * * *</item>"
            b"</scheduler_cron_time>"
            b"</agent_script_executor>"
            b"<heartbeat>"
            b"<interval_in_seconds>600</interval_in_seconds>"
            b"<miss_until_inactive>1</miss_until_inactive>"
            b"</heartbeat>"
            b"</agent_defaults>"
            b"<agent_control_defaults>"
            b"<update_to_latest>1</update_to_latest>"
            b"</agent_control_defaults>"
            b"</config_defaults>"
            b"</modify_agent_control_scan_config>"
        )

    def test_modify_agent_control_scan_config_with_missing_element_raises(self):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            # max_jitter_in_seconds is missing
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b",
                config=cfg,
            )

    def test_modify_agent_control_scan_config_missing_id_raises(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "",  # missing id
                config={
                    "config_defaults": {
                        "agent_defaults": {
                            "agent_control": {
                                "retry": {
                                    "attempts": 6,
                                    "delay_in_seconds": 60,
                                    "max_jitter_in_seconds": 10,
                                }
                            },
                            "agent_script_executor": {
                                "bulk_size": 2,
                                "bulk_throttle_time_in_ms": 300,
                                "indexer_dir_depth": 100,
                                "scheduler_cron_time": ["0 */12 * * *"],
                            },
                            "heartbeat": {
                                "interval_in_seconds": 300,
                                "miss_until_inactive": 1,
                            },
                        },
                        "agent_control_defaults": {
                            "update_to_latest": True,
                        },
                    }
                },
            )

    def test_modify_agent_control_scan_config_missing_config_raises(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b",
                config=None,  # missing config
            )

    def test_modify_agent_control_scan_config_config_not_mapping_raises(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=123
            )

    def test_modify_agent_control_scan_config_agent_control_not_mapping_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": "oops-not-a-mapping",
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_scheduler_empty_list_raises(self):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": [],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_scheduler_with_empty_item_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["", "   "],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_agent_control_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    # "agent_control": missing
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_retry_block_raises(self):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        # "retry": {}
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_retry_attempts_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            # "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_retry_delay_raises(self):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            # "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_retry_max_jitter_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            # "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_agent_script_executor_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    # "agent_script_executor": missing,
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_bulk_size_raises(self):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        # "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_bulk_throttle_time_in_ms_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        # "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_indexer_dir_depth_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        # "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_heartbeat_block_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    # "heartbeat": missing,
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_heartbeat_interval_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        # "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_heartbeat_miss_until_inactive_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        # "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_agent_control_defaults_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                # "agent_control_defaults": missing,
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_missing_update_to_latest_raises(
        self,
    ):
        cfg = {
            "config_defaults": {
                "agent_defaults": {
                    "agent_control": {
                        "retry": {
                            "attempts": 6,
                            "delay_in_seconds": 60,
                            "max_jitter_in_seconds": 10,
                        }
                    },
                    "agent_script_executor": {
                        "bulk_size": 2,
                        "bulk_throttle_time_in_ms": 300,
                        "indexer_dir_depth": 100,
                        "scheduler_cron_time": ["0 */12 * * *"],
                    },
                    "heartbeat": {
                        "interval_in_seconds": 300,
                        "miss_until_inactive": 1,
                    },
                },
                "agent_control_defaults": {
                    # "update_to_latest": True,
                },
            }
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )

    def test_modify_agent_control_scan_config_wrong_update_to_latest_raises(
        self,
    ):
        cfg = {
            "agent_defaults": {
                "agent_control": {
                    "retry": {
                        "attempts": 6,
                        "delay_in_seconds": 60,
                        "max_jitter_in_seconds": 10,
                    }
                },
                "agent_script_executor": {
                    "bulk_size": 1,
                    "bulk_throttle_time_in_ms": 100,
                    "indexer_dir_depth": 10,
                    "scheduler_cron_time": [
                        "0 23 * * *",
                        "0 22 * * *",
                    ],
                },
                "heartbeat": {
                    "interval_in_seconds": 600,
                    "miss_until_inactive": 1,
                },
            },
            "agent_control_defaults": {
                "update_to_latest": "test",
            },
        }
        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_agent_control_scan_config(
                "fb6451bf-ec5a-45a8-8bab-5cf4b862e51b", config=cfg
            )



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.audits import (
    GmpCloneAuditTestMixin,
    GmpCreateAuditTestMixin,
    GmpDeleteAuditTestMixin,
    GmpGetAuditsTestMixin,
    GmpGetAuditTestMixin,
    GmpModifyAuditTestMixin,
    GmpResumeAuditTestMixin,
    GmpStartAuditTestMixin,
    GmpStopAuditTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPCloneAuditTestCase(GmpCloneAuditTestMixin, GMPTestCase):
    pass


class GMPCreateAuditTestCase(GmpCreateAuditTestMixin, GMPTestCase):
    pass


class GMPDeleteAuditTestCase(GmpDeleteAuditTestMixin, GMPTestCase):
    pass


class GMPGetAuditTestCase(GmpGetAuditTestMixin, GMPTestCase):
    pass


class GMPGetAuditsTestCase(GmpGetAuditsTestMixin, GMPTestCase):
    pass


class GMPModifyAuditTestCase(GmpModifyAuditTestMixin, GMPTestCase):
    pass


class GMPResumeAuditTestCase(GmpResumeAuditTestMixin, GMPTestCase):
    pass


class GMPStartAuditTestCase(GmpStartAuditTestMixin, GMPTestCase):
    pass


class GMPStopAuditTestCase(GmpStopAuditTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_permission import GmpClonePermissionTestMixin
from .test_create_permission import GmpCreatePermissionTestMixin
from .test_delete_permission import GmpDeletePermissionTestMixin
from .test_get_permission import GmpGetPermissionTestMixin
from .test_get_permissions import GmpGetPermissionsTestMixin
from .test_modify_permission import GmpModifyPermissionTestMixin

__all__ = (
    "GmpClonePermissionTestMixin",
    "GmpCreatePermissionTestMixin",
    "GmpDeletePermissionTestMixin",
    "GmpGetPermissionTestMixin",
    "GmpGetPermissionsTestMixin",
    "GmpModifyPermissionTestMixin",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._entity_type import EntityType


class PermissionSubjectType(Enum):
    """Enum for permission subject type"""

    USER = "user"
    GROUP = "group"
    ROLE = "role"


class Permissions:
    @classmethod
    def clone_permission(cls, permission_id: EntityID) -> Request:
        """Clone an existing permission

        Args:
            permission_id: UUID of an existing permission to clone from
        """
        if not permission_id:
            raise RequiredArgument(
                function=cls.clone_permission.__name__,
                argument="permission_id",
            )

        cmd = XmlCommand("create_permission")
        cmd.add_element("copy", str(permission_id))
        return cmd

    @classmethod
    def create_permission(
        cls,
        name: str,
        subject_id: EntityID,
        subject_type: PermissionSubjectType | str,
        *,
        resource_id: str | None = None,
        resource_type: EntityType | str | None = None,
        comment: str | None = None,
    ) -> Request:
        """Create a new permission

        Args:
            name: Name of the new permission
            subject_id: UUID of subject to whom the permission is granted
            subject_type: Type of the subject user, group or role
            comment: Comment for the permission
            resource_id: UUID of entity to which the permission applies
            resource_type: Type of the resource. For Super permissions user,
                group or role
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_permission.__name__, argument="name"
            )

        if not subject_id:
            raise RequiredArgument(
                function=cls.create_permission.__name__, argument="subject_id"
            )

        if not isinstance(subject_type, PermissionSubjectType):
            subject_type = PermissionSubjectType(subject_type)

        cmd = XmlCommand("create_permission")
        cmd.add_element("name", name)

        xml_subject = cmd.add_element("subject", attrs={"id": str(subject_id)})
        xml_subject.add_element("type", subject_type.value)

        if comment:
            cmd.add_element("comment", comment)

        if resource_id or resource_type:
            if not resource_id:
                raise RequiredArgument(
                    function=cls.create_permission.__name__,
                    argument="resource_id",
                )

            if not resource_type:
                raise RequiredArgument(
                    function=cls.create_permission.__name__,
                    argument="resource_type",
                )

            if not isinstance(resource_type, EntityType):
                resource_type = EntityType(resource_type)

            xml_resource = cmd.add_element(
                "resource", attrs={"id": resource_id}
            )

            actual_resource_type = resource_type
            if resource_type.value == EntityType.AUDIT.value:
                actual_resource_type = EntityType.TASK
            elif resource_type.value == EntityType.POLICY.value:
                actual_resource_type = EntityType.SCAN_CONFIG

            xml_resource.add_element("type", actual_resource_type.value)

        return cmd

    @classmethod
    def delete_permission(
        cls, permission_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing permission

        Args:
            permission_id: UUID of the permission to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not permission_id:
            raise RequiredArgument(
                function=cls.delete_permission.__name__,
                argument="permission_id",
            )

        cmd = XmlCommand("delete_permission")
        cmd.set_attribute("permission_id", str(permission_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_permissions(
        *,
        filter_string: str | None = None,
        filter_id: str | None = None,
        trash: bool | None = None,
    ) -> Request:
        """Request a list of permissions

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get permissions in the trashcan instead
        """
        cmd = XmlCommand("get_permissions")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_permission(cls, permission_id: EntityID) -> Request:
        """Request a single permission

        Args:
            permission_id: UUID of an existing permission
        """
        cmd = XmlCommand("get_permissions")

        if not permission_id:
            raise RequiredArgument(
                function=cls.get_permission.__name__, argument="permission_id"
            )

        cmd.set_attribute("permission_id", str(permission_id))
        return cmd

    @classmethod
    def modify_permission(
        cls,
        permission_id: EntityID,
        *,
        comment: str | None = None,
        name: str | None = None,
        resource_id: EntityID | None = None,
        resource_type: EntityType | str | None = None,
        subject_id: EntityID | None = None,
        subject_type: PermissionSubjectType | str | None = None,
    ) -> Request:
        """Modifies an existing permission.

        Args:
            permission_id: UUID of permission to be modified.
            comment: The comment on the permission.
            name: Permission name, currently the name of a command.
            subject_id: UUID of subject to whom the permission is granted
            subject_type: Type of the subject user, group or role
            resource_id: UUID of entity to which the permission applies
            resource_type: Type of the resource. For Super permissions user,
                group or role
        """
        if not permission_id:
            raise RequiredArgument(
                function=cls.modify_permission.__name__,
                argument="permission_id",
            )

        cmd = XmlCommand("modify_permission")
        cmd.set_attribute("permission_id", str(permission_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if resource_id or resource_type:
            if not resource_id:
                raise RequiredArgument(
                    function=cls.modify_permission.__name__,
                    argument="resource_id",
                )

            if not resource_type:
                raise RequiredArgument(
                    function=cls.modify_permission.__name__,
                    argument="resource_type",
                )

            if not isinstance(resource_type, EntityType):
                resource_type = EntityType(resource_type)

            xml_resource = cmd.add_element(
                "resource", attrs={"id": str(resource_id)}
            )
            actual_resource_type = resource_type
            if resource_type.value == EntityType.AUDIT.value:
                actual_resource_type = EntityType.TASK
            elif resource_type.value == EntityType.POLICY.value:
                actual_resource_type = EntityType.SCAN_CONFIG
            xml_resource.add_element("type", actual_resource_type.value)

        if subject_id or subject_type:
            if not subject_id:
                raise RequiredArgument(
                    function=cls.modify_permission.__name__,
                    argument="subject_id",
                )

            if not subject_type:
                raise RequiredArgument(
                    function=cls.modify_permission.__name__,
                    argument="subject_type",
                )

            if not isinstance(subject_type, PermissionSubjectType):
                subject_type = PermissionSubjectType(subject_type)

            xml_subject = cmd.add_element(
                "subject", attrs={"id": str(subject_id)}
            )
            xml_subject.add_element("type", subject_type.value)

        return cmd



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .port_lists import (
    GmpClonePortListTestMixin,
    GmpCreatePortListTestMixin,
    GmpCreatePortRangeTestMixin,
    GmpDeletePortListTestMixin,
    GmpDeletePortRangeTestMixin,
    GmpGetPortListsTestMixin,
    GmpGetPortListTestMixin,
    GmpModifyPortListTestMixin,
)


class Gmpv224ClonePortListTestCase(GmpClonePortListTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreatePortListTestCase(
    GmpCreatePortListTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CreatePortRangeListTestCase(
    GmpCreatePortRangeTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224DeletePortListTestCase(
    GmpDeletePortListTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224DeletePortRangeTestCase(
    GmpDeletePortRangeTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetPortListTestCase(GmpGetPortListTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetPortListsTestCase(GmpGetPortListsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyPortListTestCase(
    GmpModifyPortListTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

import httpx

from gvm.errors import InvalidArgumentType
from gvm.protocols.http.openvasd._scans import (
    Credential,
    CredentialUP,
    ScanAction,
    ScanPreference,
    ScansAPI,
    Target,
    VTParameter,
    VTSelection,
)


def _mock_response(status_code=200, json_data=None):
    resp = MagicMock(spec=httpx.Response)
    resp.status_code = status_code
    resp.json.return_value = json_data or {}
    resp.raise_for_status = MagicMock()
    return resp


class TestScansAPI(unittest.TestCase):
    def setUp(self):
        self.mock_client = MagicMock(spec=httpx.Client)

    def test_create_scan_success(self):
        mock_resp = _mock_response()
        self.mock_client.post.return_value = mock_resp

        api = ScansAPI(self.mock_client)

        target = Target(
            hosts=["localhost"],
            credentials=[
                Credential(
                    service="ssh",
                    port=22,
                    up=CredentialUP(username="user", password="pass"),
                )
            ],
        )
        vt_selection = [
            VTSelection(
                oid="1.3.6.1.4.1.25623.1.0.100001",
                parameters=[VTParameter(id=1, value="true")],
            )
        ]
        preferences = [
            ScanPreference(id="auto_enable_dependencies", value="True")
        ]

        api.create(target, vt_selection, scan_preferences=preferences)

        self.mock_client.post.assert_called_once_with(
            "/scans",
            json={
                "target": {
                    "hosts": ["localhost"],
                    "excluded_hosts": [],
                    "ports": [],
                    "credentials": [
                        {
                            "service": "ssh",
                            "port": 22,
                            "up": {
                                "username": "user",
                                "password": "pass",
                                "privilege_username": None,
                                "privilege_password": None,
                            },
                            "krb5": None,
                            "usk": None,
                            "snmp": None,
                        }
                    ],
                    "alive_test_ports": [],
                    "alive_test_methods": [],
                    "reverse_lookup_unify": False,
                    "reverse_lookup_only": False,
                },
                "vts": [
                    {
                        "oid": "1.3.6.1.4.1.25623.1.0.100001",
                        "parameters": [{"id": 1, "value": "true"}],
                    }
                ],
                "scan_preferences": [
                    {"id": "auto_enable_dependencies", "value": "True"}
                ],
            },
        )

    def test_create_scan_without_scan_preferences(self):
        mock_resp = _mock_response()
        self.mock_client.post.return_value = mock_resp

        api = ScansAPI(self.mock_client)

        target = Target(hosts=["localhost"])
        vt_selection = [VTSelection(oid="1.3.6.1.4.1.25623.1.0.100001")]

        api.create(target, vt_selection)

        self.mock_client.post.assert_called_once_with(
            "/scans",
            json={
                "target": {
                    "hosts": ["localhost"],
                    "excluded_hosts": [],
                    "ports": [],
                    "credentials": [],
                    "alive_test_ports": [],
                    "alive_test_methods": [],
                    "reverse_lookup_unify": False,
                    "reverse_lookup_only": False,
                },
                "vts": [
                    {"oid": "1.3.6.1.4.1.25623.1.0.100001", "parameters": []}
                ],
            },
        )

    def test_create_scan_failure_raises_httpx_error(self):
        mock_response = _mock_response()
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Bad Request",
            request=MagicMock(),
            response=MagicMock(status_code=400),
        )
        self.mock_client.post.return_value = mock_response

        api = ScansAPI(self.mock_client)

        target = Target(hosts=["localhost"])
        vt_selection = [VTSelection(oid="test-oid")]
        preferences = [ScanPreference(id="test", value="4")]

        with self.assertRaises(httpx.HTTPStatusError):
            api.create(target, vt_selection, scan_preferences=preferences)

        self.mock_client.post.assert_called_once_with(
            "/scans",
            json={
                "target": {
                    "hosts": ["localhost"],
                    "excluded_hosts": [],
                    "ports": [],
                    "credentials": [],
                    "alive_test_ports": [],
                    "alive_test_methods": [],
                    "reverse_lookup_unify": False,
                    "reverse_lookup_only": False,
                },
                "vts": [{"oid": "test-oid", "parameters": []}],
                "scan_preferences": [{"id": "test", "value": "4"}],
            },
        )

    def test_create_scan_failure_httpx_error_with_suppress_exceptions(self):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Bad Request",
            request=MagicMock(),
            response=MagicMock(status_code=400),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.post.return_value = mock_response

        api = ScansAPI(self.mock_client, suppress_exceptions=True)

        target = Target(hosts=["localhost"])
        vt_selection = [VTSelection(oid="1")]
        preferences = [ScanPreference(id="test", value="4")]

        response = api.create(
            target, vt_selection, scan_preferences=preferences
        )

        self.mock_client.post.assert_called_once_with(
            "/scans",
            json={
                "target": {
                    "hosts": ["localhost"],
                    "excluded_hosts": [],
                    "ports": [],
                    "credentials": [],
                    "alive_test_ports": [],
                    "alive_test_methods": [],
                    "reverse_lookup_unify": False,
                    "reverse_lookup_only": False,
                },
                "vts": [{"oid": "1", "parameters": []}],
                "scan_preferences": [{"id": "test", "value": "4"}],
            },
        )

        self.assertEqual(response, mock_http_error.response)
        self.assertEqual(response.status_code, 400)

    def test_delete_scan_success(self):
        mock_resp = _mock_response(status_code=204)
        self.mock_client.delete.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        result = api.delete("scan-123")
        self.assertEqual(result, 204)

    def test_delete_scan_returns_500_on_httpx_error_with_suppress_exceptions(
        self,
    ):
        mock_response = MagicMock(spec=httpx.Response)
        mock_response.status_code = 500

        self.mock_client.delete.side_effect = httpx.HTTPStatusError(
            "failed", request=MagicMock(), response=mock_response
        )

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        status = api.delete("scan-1")
        self.assertEqual(status, 500)

    def test_delete_scan_raise_on_httpx_error(self):
        mock_response = MagicMock(spec=httpx.Response)
        mock_response.status_code = 500

        self.mock_client.delete.side_effect = httpx.HTTPStatusError(
            "failed", request=MagicMock(), response=mock_response
        )

        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.delete("scan-1")

    def test_get_all_scans(self):
        mock_resp = _mock_response()
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        result = api.get_all()
        self.mock_client.get.assert_called_once_with("/scans")
        self.assertEqual(result, mock_resp)

    def test_get_all_scans_failure_raises_httpx_error(self):
        mock_response = _mock_response()
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_all()

        self.mock_client.get.assert_called_once_with("/scans")

    def test_get_all_scans_failure_returns_500_with_suppress_exceptions(self):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        response = api.get_all()

        self.mock_client.get.assert_called_once_with("/scans")
        self.assertEqual(response, mock_http_error.response)
        self.assertEqual(response.status_code, 500)

    def test_get_scan_by_id(self):
        mock_resp = _mock_response()
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        result = api.get("scan-1")
        self.mock_client.get.assert_called_once_with("/scans/scan-1")
        self.assertEqual(result, mock_resp)

    def test_get_scan_by_id_failure_raises_httpx_error(self):
        mock_response = _mock_response()
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get("scan-1")

        self.mock_client.get.assert_called_once_with("/scans/scan-1")

    def test_get_scan_by_id_failure_return_500_httpx_error_with_suppress_exceptions(
        self,
    ):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        response = api.get("scan-1")

        self.mock_client.get.assert_called_once_with("/scans/scan-1")
        self.assertEqual(response, mock_http_error.response)
        self.assertEqual(response.status_code, 500)

    def test_get_results_with_range(self):
        mock_resp = _mock_response()
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        api.get_results("scan-1", range_start=5, range_end=10)
        self.mock_client.get.assert_called_once_with(
            "/scans/scan-1/results", params={"range": "5-10"}
        )

    def test_get_results_only_range_start(self):
        mock_response = _mock_response()
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        result = api.get_results("scan-1", range_start=7)
        self.mock_client.get.assert_called_once_with(
            "/scans/scan-1/results", params={"range": "7"}
        )
        self.assertEqual(result, mock_response)

    def test_get_results_only_range_end(self):
        mock_response = _mock_response()
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        result = api.get_results("scan-1", range_end=9)
        self.mock_client.get.assert_called_once_with(
            "/scans/scan-1/results", params={"range": "0-9"}
        )
        self.assertEqual(result, mock_response)

    def test_get_results_without_range(self):
        mock_response = _mock_response()
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        result = api.get_results("scan-1")
        self.mock_client.get.assert_called_once_with(
            "/scans/scan-1/results", params={}
        )
        self.assertEqual(result, mock_response)

    def test_get_results_invalid_range_type_for_range_start(self):
        api = ScansAPI(self.mock_client)
        with self.assertRaises(InvalidArgumentType):
            api.get_results("scan-1", range_start="wrong", range_end=5)

    def test_get_results_invalid_range_type_for_range_end(self):
        api = ScansAPI(self.mock_client)
        with self.assertRaises(InvalidArgumentType):
            api.get_results("scan-1", range_start=5, range_end="wrong")

    def test_get_results_invalid_range_end_type_only(self):
        api = ScansAPI(self.mock_client)
        with self.assertRaises(InvalidArgumentType):
            api.get_results("scan-1", range_end="not-an-int")

    def test_get_results_with_range_failure_raises_httpx_error(self):
        mock_response = _mock_response()
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_results("scan-1", range_start=5, range_end=10)

        self.mock_client.get.assert_called_once_with(
            "/scans/scan-1/results", params={"range": "5-10"}
        )

    def test_get_results_with_range_failure_return_500_httpx_error_with_suppress_exceptions(
        self,
    ):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        response = api.get_results("scan-1", range_start=5, range_end=10)

        self.mock_client.get.assert_called_once_with(
            "/scans/scan-1/results", params={"range": "5-10"}
        )
        self.assertEqual(response, mock_http_error.response)
        self.assertEqual(response.status_code, 500)

    def test_get_result(self):
        mock_resp = _mock_response()
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        api.get_result("scan-1", "99")
        self.mock_client.get.assert_called_once_with("/scans/scan-1/results/99")

    def test_get_result_failure_raises_httpx_error(self):
        mock_response = _mock_response()
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_result("scan-1", "99")

        self.mock_client.get.assert_called_once_with("/scans/scan-1/results/99")

    def test_get_result_failure_return_500_httpx_error_with_suppress_exceptions(
        self,
    ):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        response = api.get_result("scan-1", "99")

        self.mock_client.get.assert_called_once_with("/scans/scan-1/results/99")
        self.assertEqual(response, mock_http_error.response)
        self.assertEqual(response.status_code, 500)

    def test_get_status(self):
        mock_resp = _mock_response()
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        api.get_status("scan-1")
        self.mock_client.get.assert_called_once_with("/scans/scan-1/status")

    def test_get_status_failure_raises_httpx_error(self):
        mock_response = _mock_response()
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_status("scan-1")

        self.mock_client.get.assert_called_once_with("/scans/scan-1/status")

    def test_get_status_failure_return_500_httpx_error_with_suppress_exceptions(
        self,
    ):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        response = api.get_status("scan-1")

        self.mock_client.get.assert_called_once_with("/scans/scan-1/status")
        self.assertEqual(response, mock_http_error.response)
        self.assertEqual(response.status_code, 500)

    def test_run_action_success(self):
        mock_resp = _mock_response(status_code=202)
        self.mock_client.post.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        status = api._run_action("scan-1", ScanAction.START)
        self.assertEqual(status, 202)

    def test_run_action_failure_raise_httpx_error(self):
        self.mock_client.post.side_effect = httpx.HTTPStatusError(
            "failed", request=MagicMock(), response=MagicMock(status_code=500)
        )
        api = ScansAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api._run_action("scan-1", ScanAction.STOP)

    def test_run_action_failure_returns_500_with_suppress_exceptions(self):
        self.mock_client.post.side_effect = httpx.HTTPStatusError(
            "failed", request=MagicMock(), response=MagicMock(status_code=500)
        )

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        status = api._run_action("scan-1", ScanAction.STOP)
        self.assertEqual(status, 500)

    def test_start_scan(self):
        api = ScansAPI(self.mock_client)
        api._run_action = MagicMock(return_value=200)
        result = api.start("scan-abc")
        api._run_action.assert_called_once_with("scan-abc", ScanAction.START)
        self.assertEqual(result, 200)

    def test_start_scan_failure_raises_httpx_error(self):
        api = ScansAPI(self.mock_client)
        api._run_action = MagicMock(
            side_effect=httpx.HTTPStatusError(
                "Scan start failed",
                request=MagicMock(),
                response=MagicMock(status_code=500),
            )
        )

        with self.assertRaises(httpx.HTTPStatusError):
            api.start("scan-abc")

    def test_start_scan_failure_returns_500_with_suppress_exceptions(self):
        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        api._run_action = MagicMock(return_value=500)
        result = api.start("scan-abc")
        self.assertEqual(result, 500)

    def test_stop_scan(self):
        api = ScansAPI(self.mock_client)
        api._run_action = MagicMock(return_value=200)
        result = api.stop("scan-abc")
        api._run_action.assert_called_once_with("scan-abc", ScanAction.STOP)
        self.assertEqual(result, 200)

    def test_stop_scan_failure_raises_httpx_error(self):
        api = ScansAPI(self.mock_client)
        api._run_action = MagicMock(
            side_effect=httpx.HTTPStatusError(
                "Scan stop failed",
                request=MagicMock(),
                response=MagicMock(status_code=500),
            )
        )

        with self.assertRaises(httpx.HTTPStatusError):
            api.stop("scan-abc")

    def test_stop_scan_failure_returns_500_with_suppress_exceptions(self):
        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        api._run_action = MagicMock(return_value=500)
        result = api.stop("scan-abc")
        self.assertEqual(result, 500)

    def test_get_preferences_success(self):
        mock_resp = _mock_response(
            200,
            [
                {
                    "id": "optimize_test",
                    "name": "Optimize Test",
                    "default": True,
                    "description": "By default, optimize_test is enabled...",
                }
            ],
        )
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)
        response = api.get_preferences()

        self.mock_client.get.assert_called_once_with("/scans/preferences")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()[0]["id"], "optimize_test")

    def test_get_preferences_failure_raises_httpx_error(self):
        mock_resp = _mock_response()
        mock_resp.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server error",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client)

        with self.assertRaises(httpx.HTTPStatusError):
            api.get_preferences()

        self.mock_client.get.assert_called_once_with("/scans/preferences")

    def test_get_preferences_suppressed_exception_returns_response(self):
        mock_error_response = MagicMock(status_code=400)
        mock_resp = MagicMock()
        mock_resp.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Bad Request",
            request=MagicMock(),
            response=mock_error_response,
        )
        self.mock_client.get.return_value = mock_resp

        api = ScansAPI(self.mock_client, suppress_exceptions=True)
        response = api.get_preferences()

        self.mock_client.get.assert_called_once_with("/scans/preferences")
        self.assertEqual(response, mock_error_response)
        self.assertEqual(response.status_code, 400)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.utils import to_base64


class TestToBase64(unittest.TestCase):
    def test_to_base64(self):
        foo64 = to_base64("foo")
        self.assertEqual("Zm9v", foo64)

        bar64 = to_base64("bar")
        self.assertEqual("YmFy", bar64)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import logging

from ._connection import GvmConnection

logger = logging.getLogger("gvm.connections.debug")


class DebugConnection:
    """Wrapper around a connection for debugging purposes

    Allows to debug the connection flow including send and read data. Internally
    it uses the python `logging <https://docs.python.org/3/library/logging.html>`_
    framework to create debug messages. Please take a look at
    `the logging tutorial <https://docs.python.org/3/howto/logging.html#logging-basic-tutorial>`_
    for further details.

    Example:

    .. code-block:: python

        import logging

        logging.basicConfig(level=logging.DEBUG)

        socket_connection = UnixSocketConnection(path="/var/run/gvm.sock")
        connection = DebugConnection(socket_connection)
        gmp = GMP(connection=connection)
    """

    def __init__(self, connection: GvmConnection):
        """
        Create a new DebugConnection instance.

        Args:
            connection: GvmConnection to observe
        """
        self._connection = connection

    def read(self) -> bytes:
        data = self._connection.read()

        logger.debug("Read %s characters. Data %r", len(data), data)

        self.last_read_data = data
        return data

    def send(self, data: bytes) -> None:
        self.last_send_data = data

        logger.debug("Sending %s characters. Data %r", len(data), data)

        return self._connection.send(data)

    def connect(self) -> None:
        logger.debug("Connecting")

        return self._connection.connect()

    def disconnect(self) -> None:
        logger.debug("Disconnecting")

        return self._connection.disconnect()

    def finish_send(self) -> None:
        logger.debug("Finish send")

        self._connection.finish_send()



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tls_certificates import (
    GmpCloneTLSCertificateTestMixin,
    GmpCreateTLSCertificateTestMixin,
    GmpDeleteTLSCertificateTestMixin,
    GmpGetTLSCertificatesTestMixin,
    GmpGetTLSCertificateTestMixin,
    GmpModifyTLSCertificateTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPCloneTLSCertificateTestCase(
    GmpCloneTLSCertificateTestMixin, GMPTestCase
):
    pass


class GMPCreateTLSCertificateTestCase(
    GmpCreateTLSCertificateTestMixin, GMPTestCase
):
    pass


class GMPDeleteTLSCertificateTestCase(
    GmpDeleteTLSCertificateTestMixin, GMPTestCase
):
    pass


class GMPGetTLSCertificateTestCase(GmpGetTLSCertificateTestMixin, GMPTestCase):
    pass


class GMPGetTLSCertificatesTestCase(
    GmpGetTLSCertificatesTestMixin, GMPTestCase
):
    pass


class GMPModifyTLSCertificateTestCase(
    GmpModifyTLSCertificateTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class HostsOrdering(Enum):
    """Enum for host ordering during scans"""

    SEQUENTIAL = "sequential"
    RANDOM = "random"
    REVERSE = "reverse"


class Hosts:
    @classmethod
    def create_host(cls, name: str, *, comment: str | None = None) -> Request:
        """Create a new host host

        Args:
            name: Name for the new host host
            comment: Comment for the new host host
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_host.__name__, argument="name"
            )

        cmd = XmlCommand("create_asset")
        host = cmd.add_element("asset")
        host.add_element("type", "host")
        host.add_element("name", name)

        if comment:
            host.add_element("comment", comment)

        return cmd

    @classmethod
    def delete_host(cls, host_id: EntityID) -> Request:
        """Deletes an existing host

        Args:
            host_id: UUID of the single host to delete.
        """
        if not host_id:
            raise RequiredArgument(
                function=cls.delete_host.__name__,
                argument="host_id",
            )

        cmd = XmlCommand("delete_asset")
        cmd.set_attribute("asset_id", str(host_id))

        return cmd

    @staticmethod
    def get_hosts(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of hosts

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: Whether to include additional information (e.g. tags)
        """

        cmd = XmlCommand("get_assets")

        cmd.set_attribute("type", "host")

        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_host(
        cls, host_id: EntityID, *, details: bool | None = None
    ) -> Request:
        """Request a single host

        Arguments:
            host_id: UUID of an existing host
            details: Whether to include additional information (e.g. tags)
        """
        cmd = XmlCommand("get_assets")

        if not host_id:
            raise RequiredArgument(
                function=cls.get_host.__name__, argument="host_id"
            )

        cmd.set_attribute("asset_id", str(host_id))
        cmd.set_attribute("type", "host")

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def modify_host(
        cls, host_id: EntityID, *, comment: str | None = None
    ) -> Request:
        """Modifies an existing host.

        Args:
            host_id: UUID of the host to be modified.
            comment: Comment for the host. Not passing a comment
                arguments clears the comment for this host.
        """
        if not host_id:
            raise RequiredArgument(
                function=cls.modify_host.__name__, argument="host_id"
            )

        cmd = XmlCommand("modify_asset")
        cmd.set_attribute("asset_id", str(host_id))
        if not comment:
            comment = ""
        cmd.add_element("comment", comment)

        return cmd



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.protocols.core._connection import XmlReader


class XmlReaderTestCase(unittest.TestCase):
    def test_is_end_xml_false(self):
        reader = XmlReader()
        reader.start_xml()

        false = reader.is_end_xml()
        self.assertFalse(false)



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.system.feed import GmpGetFeedsTestMixin, GmpGetFeedTestMixin
from ...gmpv225 import GMPTestCase


class Gmpv225GetFeedTestCase(GmpGetFeedTestMixin, GMPTestCase):
    pass


class Gmpv225GetFeedsTestCase(GmpGetFeedsTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.utils import to_comma_list


class TestToCoCommaList(unittest.TestCase):
    def test_to_comma_list(self):
        # pylint: disable=invalid-name
        the_list = ["foo", "bar", "baz"]

        comma_string = to_comma_list(the_list)

        self.assertEqual("foo,bar,baz", comma_string)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import PortLists, PortRangeType


class PortListsTestCase(unittest.TestCase):
    def test_clone(self) -> None:
        request = PortLists.clone_port_list("a1")

        self.assertEqual(
            bytes(request),
            b"<create_port_list><copy>a1</copy></create_port_list>",
        )

    def test_clone_missing_id(self):
        with self.assertRaises(RequiredArgument):
            PortLists.clone_port_list("")

        with self.assertRaises(RequiredArgument):
            PortLists.clone_port_list(None)

    def test_create_port_list_missing_name(self):
        with self.assertRaises(RequiredArgument):
            PortLists.create_port_list(name=None, port_range="T:1-1234")

        with self.assertRaises(RequiredArgument):
            PortLists.create_port_list(name="", port_range="T:1-1234")

    def test_create_port_list_missing_port_range(self):
        with self.assertRaises(RequiredArgument):
            PortLists.create_port_list(name="foo", port_range=None)

        with self.assertRaises(RequiredArgument):
            PortLists.create_port_list(name="foo", port_range="")

    def test_create_port_list(self):
        request = PortLists.create_port_list(name="foo", port_range="T:1-1234")

        self.assertEqual(
            bytes(request),
            b"<create_port_list>"
            b"<name>foo</name>"
            b"<port_range>T:1-1234</port_range>"
            b"</create_port_list>",
        )

    def test_create_port_list_with_comment(self):
        request = PortLists.create_port_list(
            name="foo", port_range="T:1-1234", comment="lorem"
        )

        self.assertEqual(
            bytes(request),
            b"<create_port_list>"
            b"<name>foo</name>"
            b"<port_range>T:1-1234</port_range>"
            b"<comment>lorem</comment>"
            b"</create_port_list>",
        )

    def test_create_port_range_missing_port_list_id(self):
        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id=None,
                start=1,
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="",
                start=1,
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

    def test_create_port_range_missing_start(self):
        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="pl1",
                start=None,
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="pl1",
                start="",
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

    def test_create_port_range_missing_end(self):
        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="pl1",
                start=1,
                end=None,
                port_range_type=PortRangeType.TCP,
            )

        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="pl1",
                start=1,
                end="",
                port_range_type=PortRangeType.TCP,
            )

    def test_create_port_range_missing_port_range_type(self):
        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="pl1", start=1, end=1234, port_range_type=None
            )

        with self.assertRaises(RequiredArgument):
            PortLists.create_port_range(
                port_list_id="pl1", start=1, end=1234, port_range_type=""
            )

    def test_create_port_range_invalid_port_range_type(self):
        with self.assertRaises(InvalidArgument):
            PortLists.create_port_range(
                port_list_id="pl1", start=1, end=1234, port_range_type="blubb"
            )

    def test_create_port_range(self):
        request = PortLists.create_port_range(
            port_list_id="pl1",
            start=1,
            end=1234,
            port_range_type=PortRangeType.TCP,
        )

        self.assertEqual(
            bytes(request),
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>TCP</type>"
            b"</create_port_range>",
        )

        request = PortLists.create_port_range(
            port_list_id="pl1",
            start=1,
            end=1234,
            port_range_type=PortRangeType.UDP,
        )

        self.assertEqual(
            bytes(request),
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>UDP</type>"
            b"</create_port_range>",
        )

        request = PortLists.create_port_range(
            port_list_id="pl1",
            start="1",
            end="1234",
            port_range_type=PortRangeType.TCP,
        )

        self.assertEqual(
            bytes(request),
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>TCP</type>"
            b"</create_port_range>",
        )

    def test_create_port_range_with_comment(self):
        request = PortLists.create_port_range(
            port_list_id="pl1",
            start=1,
            end=1234,
            port_range_type=PortRangeType.TCP,
            comment="lorem",
        )

        self.assertEqual(
            bytes(request),
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>TCP</type>"
            b"<comment>lorem</comment>"
            b"</create_port_range>",
        )

    def test_delete(self):
        request = PortLists.delete_port_list("a1")

        self.assertEqual(
            bytes(request),
            b'<delete_port_list port_list_id="a1" ultimate="0"/>',
        )

    def test_delete_ultimate(self):
        request = PortLists.delete_port_list("a1", ultimate=True)

        self.assertEqual(
            bytes(request),
            b'<delete_port_list port_list_id="a1" ultimate="1"/>',
        )

    def test_delete_missing_id(self):
        with self.assertRaises(RequiredArgument):
            PortLists.delete_port_list(None)

        with self.assertRaises(RequiredArgument):
            PortLists.delete_port_list("")

    def test_delete_port_range(self):
        request = PortLists.delete_port_range("a1")

        self.assertEqual(
            bytes(request), b'<delete_port_range port_range_id="a1"/>'
        )

    def test_delete_port_range_missing_id(self):
        with self.assertRaises(RequiredArgument):
            PortLists.delete_port_range(None)

        with self.assertRaises(RequiredArgument):
            PortLists.delete_port_range("")

    def test_get_port_lists(self):
        request = PortLists.get_port_lists()

        self.assertEqual(bytes(request), b"<get_port_lists/>")

    def test_get_port_lists_with_filter_string(self):
        request = PortLists.get_port_lists(filter_string="foo=bar")

        self.assertEqual(bytes(request), b'<get_port_lists filter="foo=bar"/>')

    def test_get_port_lists_with_filter_id(self):
        request = PortLists.get_port_lists(filter_id="f1")

        self.assertEqual(bytes(request), b'<get_port_lists filt_id="f1"/>')

    def test_get_port_lists_with_trash(self):
        request = PortLists.get_port_lists(trash=True)

        self.assertEqual(bytes(request), b'<get_port_lists trash="1"/>')

        request = PortLists.get_port_lists(trash=False)

        self.assertEqual(bytes(request), b'<get_port_lists trash="0"/>')

    def test_get_port_lists_with_details(self):
        request = PortLists.get_port_lists(details=True)

        self.assertEqual(bytes(request), b'<get_port_lists details="1"/>')

        request = PortLists.get_port_lists(details=False)

        self.assertEqual(bytes(request), b'<get_port_lists details="0"/>')

    def test_get_port_lists_with_targets(self):
        request = PortLists.get_port_lists(targets=True)

        self.assertEqual(bytes(request), b'<get_port_lists targets="1"/>')

        request = PortLists.get_port_lists(targets=False)

        self.assertEqual(bytes(request), b'<get_port_lists targets="0"/>')

    def test_get_port_list(self):
        request = PortLists.get_port_list(port_list_id="port_list_id")

        self.assertEqual(
            bytes(request),
            b'<get_port_lists port_list_id="port_list_id" details="1"/>',
        )

    def test_get_port_list_missing_port_list_id(self):
        with self.assertRaises(RequiredArgument):
            PortLists.get_port_list(port_list_id=None)

        with self.assertRaises(RequiredArgument):
            PortLists.get_port_list(port_list_id="")

        with self.assertRaises(RequiredArgument):
            PortLists.get_port_list("")

    def test_modify_port_list(self):
        request = PortLists.modify_port_list(port_list_id="p1")

        self.assertEqual(
            bytes(request), b'<modify_port_list port_list_id="p1"/>'
        )

    def test_modify_port_list_missing_port_list_id(self):
        with self.assertRaises(RequiredArgument):
            PortLists.modify_port_list(port_list_id=None)

        with self.assertRaises(RequiredArgument):
            PortLists.modify_port_list(port_list_id="")

        with self.assertRaises(RequiredArgument):
            PortLists.modify_port_list("")

    def test_modify_port_list_with_comment(self):
        request = PortLists.modify_port_list(port_list_id="p1", comment="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_port_list port_list_id="p1">'
            b"<comment>foo</comment>"
            b"</modify_port_list>",
        )

    def test_modify_port_list_with_name(self):
        request = PortLists.modify_port_list(port_list_id="p1", name="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_port_list port_list_id="p1">'
            b"<name>foo</name>"
            b"</modify_port_list>",
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .report_formats import (
    GmpCloneReportFormatTestMixin,
    GmpDeleteReportFormatTestMixin,
    GmpGetReportFormatsTestMixin,
    GmpGetReportFormatTestMixin,
    GmpImportReportFormatTestMixin,
    GmpModifyReportFormatTestMixin,
    GmpVerifyReportFormatTestMixin,
)


class Gmpv224DeleteReportFormatTestCase(
    GmpDeleteReportFormatTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetReportFormatTestCase(
    GmpGetReportFormatTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetReportFormatsTestCase(
    GmpGetReportFormatsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CloneReportFormatTestCase(
    GmpCloneReportFormatTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ImportReportFormatTestCase(
    GmpImportReportFormatTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyReportFormatTestCase(
    GmpModifyReportFormatTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224VerifyReportFormatTestCase(
    GmpVerifyReportFormatTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetGroupsTestMixin:
    def test_get_groups(self):
        self.gmp.get_groups()

        self.connection.send.has_been_called_with(b"<get_groups/>")

    def test_get_groups_with_filter_string(self):
        self.gmp.get_groups(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_groups filter="foo=bar"/>'
        )

    def test_get_groups_with_filter_id(self):
        self.gmp.get_groups(filter_id="f1")

        self.connection.send.has_been_called_with(b'<get_groups filt_id="f1"/>')

    def test_get_groups_with_trash(self):
        self.gmp.get_groups(trash=True)

        self.connection.send.has_been_called_with(b'<get_groups trash="1"/>')

        self.gmp.get_groups(trash=False)

        self.connection.send.has_been_called_with(b'<get_groups trash="0"/>')



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv226 import GMPTestCase
from .reports import (
    GmpDeleteReportTestMixin,
    GmpGetReportsTestMixin,
    GmpGetReportTestMixin,
    GmpImportReportTestMixin,
)


class GMPDeleteReportTestCase(GmpDeleteReportTestMixin, GMPTestCase):
    pass


class GMPGetReportTestCase(GmpGetReportTestMixin, GMPTestCase):
    pass


class GMPGetReportsTestCase(GmpGetReportsTestMixin, GMPTestCase):
    pass


class GMPImportReportTestCase(GmpImportReportTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import UserSettings


class UserSettingsTestCase(unittest.TestCase):
    def test_get_user_setting(self):
        request = UserSettings.get_user_setting("id")

        self.assertEqual(bytes(request), b'<get_settings setting_id="id"/>')

    def test_get_user_setting_missing_setting_id(self):
        with self.assertRaises(RequiredArgument):
            UserSettings.get_user_setting(setting_id=None)

        with self.assertRaises(RequiredArgument):
            UserSettings.get_user_setting("")

    def test_get_user_settings(self):
        request = UserSettings.get_user_settings()

        self.assertEqual(bytes(request), b"<get_settings/>")

    def test_get_user_settings_with_filter_string(self):
        request = UserSettings.get_user_settings(filter_string="foo=bar")

        self.assertEqual(bytes(request), b'<get_settings filter="foo=bar"/>')

    def test_modify_user_setting(self):
        request = UserSettings.modify_user_setting(
            setting_id="id", value="value"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_setting setting_id="id"><value>dmFsdWU=</value></modify_setting>',
        )

        request = UserSettings.modify_user_setting(name="name", value="value")

        self.assertEqual(
            bytes(request),
            b"<modify_setting><name>name</name><value>dmFsdWU=</value></modify_setting>",
        )

        request = UserSettings.modify_user_setting(name="name", value="")

        self.assertEqual(
            bytes(request),
            b"<modify_setting><name>name</name><value></value></modify_setting>",
        )

    def test_modify_user_setting_missing_setting_id(self):
        with self.assertRaises(RequiredArgument):
            UserSettings.modify_user_setting(setting_id=None, value="value")

        with self.assertRaises(RequiredArgument):
            UserSettings.modify_user_setting(setting_id="", value="value")

    def test_modify_setting_missing_name(self):
        with self.assertRaises(RequiredArgument):
            UserSettings.modify_user_setting(name=None, value="value")

        with self.assertRaises(RequiredArgument):
            UserSettings.modify_user_setting(name="", value="value")

    def test_modify_user_setting_missing_value(self):
        with self.assertRaises(RequiredArgument):
            UserSettings.modify_user_setting(setting_id="id", value=None)



# SPDX-FileCopyrightText: 2021-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v227 import ScannerType


class GmpCreateScannerTestMixin:
    def test_create_scanner(self):
        self.gmp.create_scanner(
            name="foo",
            host="localhost",
            port=1234,
            scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
            credential_id="c1",
        )

        self.connection.send.has_been_called_with(
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>2</type>"
            b'<credential id="c1"/>'
            b"</create_scanner>"
        )

    def test_create_scanner_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name=None,
                host="localhost",
                port=1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="",
                host="localhost",
                port=1234,
                scanner_type="2",
                credential_id="c1",
            )

    def test_create_scanner_missing_host(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host=None,
                port=1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="",
                port=1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

    def test_create_scanner_missing_port(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=None,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port="",
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="c1",
            )

    def test_create_scanner_missing_scanner_type(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type=None,
                credential_id="c1",
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type="",
                credential_id="c1",
            )

    def test_create_scanner_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id=None,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
                credential_id="",
            )

    def test_create_scanner_invalid_scanner_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type="bar",
                credential_id="c1",
            )

        with self.assertRaises(AttributeError):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type=ScannerType.FOO,  # pylint: disable=no-member
                credential_id="c1",
            )

        with self.assertRaises(InvalidArgument):
            self.gmp.create_scanner(
                name="foo",
                host="localhost",
                port=1234,
                scanner_type="55",
                credential_id="c1",
            )

    def test_create_scanner_with_ca_pub(self):
        self.gmp.create_scanner(
            name="foo",
            host="localhost",
            port=1234,
            ca_pub="foo",
            scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
            credential_id="c1",
        )

        self.connection.send.has_been_called_with(
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>2</type>"
            b'<credential id="c1"/>'
            b"<ca_pub>foo</ca_pub>"
            b"</create_scanner>"
        )

    def test_create_scanner_with_comment(self):
        self.gmp.create_scanner(
            name="foo",
            host="localhost",
            port=1234,
            scanner_type=ScannerType.OPENVAS_SCANNER_TYPE,
            credential_id="c1",
            comment="bar",
        )

        self.connection.send.has_been_called_with(
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>2</type>"
            b'<credential id="c1"/>'
            b"<comment>bar</comment>"
            b"</create_scanner>"
        )

    def test_create_scanner_with_openvasd_type(self):
        self.gmp.create_scanner(
            name="foo",
            host="localhost",
            port=1234,
            scanner_type=ScannerType.OPENVASD_SCANNER_TYPE,
            credential_id="c1",
        )

        self.connection.send.has_been_called_with(
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>localhost</host>"
            b"<port>1234</port>"
            b"<type>6</type>"
            b'<credential id="c1"/>'
            b"</create_scanner>"
        )

    def test_create_scanner_with_relay_host(self):
        self.gmp.create_scanner(
            name="foo",
            host="remotehost",
            port=1234,
            scanner_type=ScannerType.OPENVASD_SCANNER_TYPE,
            credential_id="c1",
            relay_host="localhost",
        )

        self.connection.send.has_been_called_with(
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>remotehost</host>"
            b"<port>1234</port>"
            b"<type>6</type>"
            b'<credential id="c1"/>'
            b"<relay_host>localhost</relay_host>"
            b"</create_scanner>"
        )

    def test_create_scanner_with_relay_port(self):
        self.gmp.create_scanner(
            name="foo",
            host="remotehost",
            port=1234,
            scanner_type=ScannerType.OPENVASD_SCANNER_TYPE,
            credential_id="c1",
            relay_port=2345,
        )

        self.connection.send.has_been_called_with(
            b"<create_scanner>"
            b"<name>foo</name>"
            b"<host>remotehost</host>"
            b"<port>1234</port>"
            b"<type>6</type>"
            b'<credential id="c1"/>'
            b"<relay_port>2345</relay_port>"
            b"</create_scanner>"
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_resource_name import GmpGetResourceNameTestMixin
from .test_get_resource_names_list import GmpGetResourceNamesListTestMixin

__all__ = ("GmpGetResourceNameTestMixin", "GmpGetResourceNamesListTestMixin")



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument

ICAL = """
BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//Greenbone.net//NONSGML Greenbone Security Manager 8.0.0//EN
BEGIN:VEVENT
UID:c35f82f1-7798-4b84-b2c4-761a33068956
DTSTAMP:20190715T124352Z
DTSTART:20190716T040000Z
END:VEVENT
END:VCALENDAR
"""


class GmpModifyScheduleTestMixin:
    def test_missing_schedule_id(self):
        with self.assertRaises(RequiredArgument) as cm:
            self.gmp.modify_schedule(schedule_id=None)

        ex = cm.exception
        self.assertEqual(ex.argument, "schedule_id")

        with self.assertRaises(RequiredArgument) as cm:
            self.gmp.modify_schedule(schedule_id="")

        ex = cm.exception
        self.assertEqual(ex.argument, "schedule_id")

    def test_modify_schedule_with_name(self):
        self.gmp.modify_schedule(schedule_id="s1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_schedule schedule_id="s1">'
            b"<name>foo</name>"
            b"</modify_schedule>"
        )

    def test_modify_schedule_with_comment(self):
        self.gmp.modify_schedule(schedule_id="s1", comment="bar")

        self.connection.send.has_been_called_with(
            b'<modify_schedule schedule_id="s1">'
            b"<comment>bar</comment>"
            b"</modify_schedule>"
        )

    def test_modify_schedule_with_icalendar(self):
        self.gmp.modify_schedule(schedule_id="s1", icalendar=ICAL)

        self.connection.send.has_been_called_with(
            '<modify_schedule schedule_id="s1">'
            f"<icalendar>{ICAL}</icalendar>"
            "</modify_schedule>".encode()
        )

    def test_modify_schedule_with_timezone(self):
        self.gmp.modify_schedule(schedule_id="s1", timezone="Europe/Berlin")

        self.connection.send.has_been_called_with(
            b'<modify_schedule schedule_id="s1">'
            b"<timezone>Europe/Berlin</timezone>"
            b"</modify_schedule>"
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=arguments-differ, redefined-builtin

import base64
import logging
import re
import warnings
from collections.abc import Callable, Iterable
from functools import wraps
from typing import (
    Any,
    Protocol,
)

from gvm.xml import XmlCommand, XmlError, parse_xml

logger = logging.getLogger(__name__)


class TypesDict(dict):
    """For dot.notation access to dictionary attributes"""

    __getattr__ = dict.get
    __setattr__ = dict.__setitem__  # type: ignore[assignment]
    __delattr__ = dict.__delitem__  # type: ignore[assignment]


def deprecation(message: str):
    warnings.warn(message, DeprecationWarning, stacklevel=2)


def deprecated(
    _func_or_cls: str | Callable | type | None = None,
    *,
    since: str | None = None,
    reason: str | None = None,
):
    """
    A decorator to mark functions, classes and methods as deprecated

    Args:
        since: An optional version since the referenced item is deprecated.
        reason: An optional reason why the references item is deprecated.

    Examples:
        .. code-block:: python

            from gvm.utils import deprecated


            @deprecated
            def my_function(*args, **kwargs): ...


            @deprecated("The function is obsolete. Please use my_func instead.")
            def my_function(*args, **kwargs): ...


            @deprecated(
                since="1.2.3",
                reason="The function is obsolete. Please use my_func instead.",
            )
            def my_function(*args, **kwargs): ...


            @deprecated(reason="The class will be removed in version 3.4.5")
            class Foo: ...


            class Foo:
                @deprecated(since="2.3.4")
                def bar(self, *args, **kwargs): ...
    """
    if isinstance(_func_or_cls, str):
        reason = _func_or_cls
        _func_or_cls = None

    def decorator_repeat(func_or_cls):
        module = func_or_cls.__module__
        name = func_or_cls.__name__

        if module == "__main__":
            msg = f"{name} is deprecated."
        else:
            msg = f"{module}.{name} is deprecated."

        if since:
            msg += f" It is deprecated since version {since}."
        if reason:
            msg += f" {reason}"

        @wraps(func_or_cls)
        def wrapper(*args, **kwargs):
            warnings.warn(msg, category=DeprecationWarning, stacklevel=3)
            return func_or_cls(*args, **kwargs)

        return wrapper

    if _func_or_cls is None:
        return decorator_repeat
    else:
        return decorator_repeat(_func_or_cls)


def check_command_status(xml: str) -> bool:
    """Check gmp response

    Look into the gmp response and check for the status in the root element

    Arguments:
        xml: XML-Source

    Returns:
        True if valid, otherwise False
    """

    if xml == 0 or xml is None:
        logger.error("XML Command is empty.")
        return False

    try:
        root = parse_xml(xml)
        status = root.attrib["status"]
        return status is not None and status[0] == "2"
    except KeyError:
        logger.error("Not received an status code within the response.")
        return False
    except XmlError as e:
        logger.error("Error while parsing the command status: %s", e)
        return False


def to_dotted_types_dict(types: list) -> TypesDict:
    """Create a dictionary accessible via dot notation"""
    dic = {}
    for typ in types:
        dic[typ.__name__] = typ
    return TypesDict(dic)


def to_bool(value: bool | int | None) -> str:
    return "1" if value else "0"


def to_base64(value: str) -> str:
    return base64.b64encode(value.encode("utf-8")).decode(encoding="utf-8")


class SupportsStr(Protocol):
    def __str__(self) -> str: ...


def to_comma_list(value: Iterable[SupportsStr]) -> str:
    return ",".join([str(value) for value in value])


@deprecated(since="24.3.0", reason="Please use XmlCommand.add_filter instead.")
def add_filter(
    cmd: XmlCommand, filter_string: str | None, filter_id: str | None
) -> None:
    cmd.add_filter(filter_string, filter_id)


def is_list_like(value: Any) -> bool:
    return isinstance(value, (list, tuple))


def check_port(value: str) -> bool:
    pattern = re.compile(
        r"^(cpe:[^\s]+|(general|[1-9][0-9]{0,4})/[0-9A-Za-z]+)$"
    )
    return bool(pattern.fullmatch(value))



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv226.entities.audit_reports import (
    GmpDeleteAuditReportTestMixin,
    GmpGetAuditReportsTestMixin,
    GmpGetAuditReportTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPDeleteAuditReportTestCase(GmpDeleteAuditReportTestMixin, GMPTestCase):
    pass


class GMPGetAuditReportTestCase(GmpGetAuditReportTestMixin, GMPTestCase):
    pass


class GMPGetAuditReportsTestCase(GmpGetAuditReportsTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""Latest supported protocols, including unstable ones.

This module exposes the latest supported protocols of GVM including versions
not yet released as stable.

The provided Gmp class implements the latest `Greenbone Management Protocol`.
The provided Osp class implements the latest Open Scanner Protocol.

For details about the possible supported protocol versions please take a look at
:py:mod:`gvm.protocols`.

Exports:
  - :py:class:`gvm.protocols.gmp.GMPv227`
  - :py:class:`gvm.protocols.ospv1.Osp`

.. _Greenbone Management Protocol:
    https://docs.greenbone.net/API/GMP/gmp.html
"""

from .gmp import (
    GMPv227 as GMP,  # noqa: N817
)
from .ospv1 import Osp

Gmp = GMP  # for backwards compatibility

__all__ = [
    "GMP",
    "Gmp",
    "Osp",
]



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetTargetsTestMixin:
    def test_get_targets(self):
        self.gmp.get_targets()

        self.connection.send.has_been_called_with(b"<get_targets/>")

    def test_get_targets_with_filter_string(self):
        self.gmp.get_targets(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_targets filter="foo=bar"/>'
        )

    def test_get_targets_with_filter_id(self):
        self.gmp.get_targets(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_targets filt_id="f1"/>'
        )

    def test_get_targets_with_trash(self):
        self.gmp.get_targets(trash=True)

        self.connection.send.has_been_called_with(b'<get_targets trash="1"/>')

        self.gmp.get_targets(trash=False)

        self.connection.send.has_been_called_with(b'<get_targets trash="0"/>')

    def test_get_targets_with_tasks(self):
        self.gmp.get_targets(tasks=True)

        self.connection.send.has_been_called_with(b'<get_targets tasks="1"/>')

        self.gmp.get_targets(tasks=False)

        self.connection.send.has_been_called_with(b'<get_targets tasks="0"/>')



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_alert import GmpCloneAlertTestMixin
from .test_create_alert import GmpCreateAlertTestMixin
from .test_delete_alert import GmpDeleteAlertTestMixin
from .test_get_alert import GmpGetAlertTestMixin
from .test_get_alerts import GmpGetAlertsTestMixin
from .test_modify_alert import GmpModifyAlertTestMixin
from .test_test_alert import GmpTestAlertTestMixin
from .test_trigger_alert import GmpTriggerAlertTestMixin

__all__ = (
    "GmpCloneAlertTestMixin",
    "GmpCreateAlertTestMixin",
    "GmpDeleteAlertTestMixin",
    "GmpGetAlertTestMixin",
    "GmpGetAlertsTestMixin",
    "GmpModifyAlertTestMixin",
    "GmpTestAlertTestMixin",
    "GmpTriggerAlertTestMixin",
)



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.system.versions import GmpGetVersionTestCase
from ...gmpv225 import GMPTestCase
from .versions import GmpGetProtocolVersionTestCase


class Gmpv225GetVersionCommandTestCase(GmpGetVersionTestCase, GMPTestCase):
    pass


class Gmpv225GmpGetProtocolVersionTestCase(
    GmpGetProtocolVersionTestCase, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import EntityType


class GmpCreateTagTestMixin:
    def test_create_tag_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_tag(
                name=None, resource_ids=["foo"], resource_type=EntityType.TASK
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_tag(
                name="", resource_ids=["foo"], resource_type=EntityType.TASK
            )

    def test_create_tag_missing_resource_filter_and_ids(self):
        self.gmp.create_tag(
            name="foo",
            resource_filter=None,
            resource_ids=None,
            resource_type=EntityType.TASK,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

        self.gmp.create_tag(
            name="foo",
            resource_filter=None,
            resource_ids=[],
            resource_type=EntityType.TASK,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

        self.gmp.create_tag(name="foo", resource_type=EntityType.TASK)

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

    def test_create_tag_both_resource_filter_and_ids(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_tag(
                name="foo",
                resource_filter="name=foo",
                resource_ids=["foo"],
                resource_type=EntityType.TASK,
            )

    def test_create_tag_invalid_resource_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_tag(
                name="foo",
                resource_type="Foo",
                resource_filter=None,
                resource_ids=["foo"],
            )

    def test_create_tag_missing_resource_type(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_tag(
                name="foo",
                resource_type=None,
                resource_filter=None,
                resource_ids=["foo"],
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_tag(
                name="foo",
                resource_type=None,
                resource_filter="name=foo",
                resource_ids=None,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_tag(
                name="foo", resource_type="", resource_ids=["foo"]
            )

    def test_create_tag_with_resource_filter(self):
        self.gmp.create_tag(
            name="foo",
            resource_filter="name=foo",
            resource_type=EntityType.TASK,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b'<resources filter="name=foo">'
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

    def test_create_tag_with_resource_filter_audit(self):
        self.gmp.create_tag(
            name="foo",
            resource_filter="name=foo",
            resource_type=EntityType.AUDIT,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b'<resources filter="name=foo">'
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

    def test_create_tag_with_resource_filter_policy(self):
        self.gmp.create_tag(
            name="foo",
            resource_filter="name=foo",
            resource_type=EntityType.POLICY,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b'<resources filter="name=foo">'
            b"<type>config</type>"
            b"</resources>"
            b"</create_tag>"
        )

    def test_create_tag_with_resource_ids(self):
        self.gmp.create_tag(
            name="foo", resource_ids=["foo"], resource_type=EntityType.TASK
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b'<resource id="foo"/>'
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

        self.gmp.create_tag(
            name="foo",
            resource_ids=["foo", "bar"],
            resource_type=EntityType.TASK,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b'<resource id="foo"/>'
            b'<resource id="bar"/>'
            b"<type>task</type>"
            b"</resources>"
            b"</create_tag>"
        )

    def test_create_tag_with_comment(self):
        self.gmp.create_tag(
            name="foo",
            resource_ids=["foo"],
            resource_type=EntityType.TASK,
            comment="bar",
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b'<resource id="foo"/>'
            b"<type>task</type>"
            b"</resources>"
            b"<comment>bar</comment>"
            b"</create_tag>"
        )

    def test_create_tag_with_value(self):
        self.gmp.create_tag(
            name="foo",
            resource_ids=["foo"],
            resource_type=EntityType.TASK,
            value="bar",
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b'<resource id="foo"/>'
            b"<type>task</type>"
            b"</resources>"
            b"<value>bar</value>"
            b"</create_tag>"
        )

    def test_create_tag_with_active(self):
        self.gmp.create_tag(
            name="foo",
            resource_ids=["foo"],
            resource_type=EntityType.TASK,
            active=True,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b'<resource id="foo"/>'
            b"<type>task</type>"
            b"</resources>"
            b"<active>1</active>"
            b"</create_tag>"
        )

        self.gmp.create_tag(
            name="foo",
            resource_ids=["foo"],
            resource_type=EntityType.TASK,
            active=False,
        )

        self.connection.send.has_been_called_with(
            b"<create_tag>"
            b"<name>foo</name>"
            b"<resources>"
            b'<resource id="foo"/>'
            b"<type>task</type>"
            b"</resources>"
            b"<active>0</active>"
            b"</create_tag>"
        )



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later

from .test_create_credential_store_credential import (
    GmpCreateCredentialStoreCredentialTestMixin,
)
from .test_modify_credential_store_credential import (
    GmpModifyCredentialStoreCredentialTestMixin,
)

__all__ = (
    "GmpCreateCredentialStoreCredentialTestMixin",
    "GmpModifyCredentialStoreCredentialTestMixin",
)



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.connections import DebugConnection, GvmConnection
from gvm.connections._connection import AbstractGvmConnection


class TestConnection(AbstractGvmConnection):
    def connect(self) -> None:
        pass


class DebugConnectionTestCase(unittest.TestCase):
    def test_is_gvm_connection(self):
        connection = DebugConnection(TestConnection())
        self.assertTrue(isinstance(connection, GvmConnection))



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetHostTestMixin:
    def test_get_host(self):
        self.gmp.get_host("a1")

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="host"/>'
        )

        self.gmp.get_host(host_id="a1")

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="host"/>'
        )

    def test_get_host_details(self):
        self.gmp.get_host("a1", details=True)

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="host" details="1"/>'
        )

        self.gmp.get_host("a1", details=False)

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="host" details="0"/>'
        )

    def test_get_host_missing_host_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_host(
                host_id=None,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.get_host(
                host_id="",
            )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyPortListTestMixin:
    def test_modify_port_list(self):
        self.gmp.modify_port_list(port_list_id="p1")

        self.connection.send.has_been_called_with(
            b'<modify_port_list port_list_id="p1"/>'
        )

    def test_modify_port_list_missing_port_list_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_port_list(port_list_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_port_list(port_list_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_port_list("")

    def test_modify_port_list_with_comment(self):
        self.gmp.modify_port_list(port_list_id="p1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_port_list port_list_id="p1">'
            b"<comment>foo</comment>"
            b"</modify_port_list>"
        )

    def test_modify_port_list_with_name(self):
        self.gmp.modify_port_list(port_list_id="p1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_port_list port_list_id="p1">'
            b"<name>foo</name>"
            b"</modify_port_list>"
        )



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetTicketsTestMixin:
    def test_get_tickets(self):
        self.gmp.get_tickets()

        self.connection.send.has_been_called_with(b"<get_tickets/>")

    def test_get_tickets_with_filter_string(self):
        self.gmp.get_tickets(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_tickets filter="foo=bar"/>'
        )

    def test_get_tickets_with_filter_id(self):
        self.gmp.get_tickets(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_tickets filt_id="f1"/>'
        )

    def test_get_tickets_with_trash(self):
        self.gmp.get_tickets(trash=True)

        self.connection.send.has_been_called_with(b'<get_tickets trash="1"/>')

        self.gmp.get_tickets(trash=False)

        self.connection.send.has_been_called_with(b'<get_tickets trash="0"/>')



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyRoleTestMixin:
    def test_modify_role(self):
        self.gmp.modify_role(role_id="r1")

        self.connection.send.has_been_called_with(
            b'<modify_role role_id="r1"/>'
        )

    def test_modify_role_missing_role_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_role(role_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_role(role_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_role("")

    def test_modify_role_with_comment(self):
        self.gmp.modify_role(role_id="r1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_role role_id="r1"><comment>foo</comment></modify_role>'
        )

    def test_modify_role_with_name(self):
        self.gmp.modify_role(role_id="r1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_role role_id="r1"><name>foo</name></modify_role>'
        )

    def test_modify_role_with_users(self):
        self.gmp.modify_role(role_id="r1", users=[])

        self.connection.send.has_been_called_with(
            b'<modify_role role_id="r1"/>'
        )

        self.gmp.modify_role(role_id="r1", users=["foo"])

        self.connection.send.has_been_called_with(
            b'<modify_role role_id="r1"><users>foo</users></modify_role>'
        )

        self.gmp.modify_role(role_id="r1", users=["foo", "bar"])

        self.connection.send.has_been_called_with(
            b'<modify_role role_id="r1"><users>foo,bar</users></modify_role>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.system.feed import GmpGetFeedsTestMixin, GmpGetFeedTestMixin
from ...gmpv226 import GMPTestCase


class GMPGetFeedTestCase(GmpGetFeedTestMixin, GMPTestCase):
    pass


class GMPGetFeedsTestCase(GmpGetFeedsTestMixin, GMPTestCase):
    pass



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later


class GmpGetAgentInstallersTestMixin:
    def test_get_agent_installers(self):
        self.gmp.get_agent_installers()

        self.connection.send.has_been_called_with(b"<get_agent_installers/>")

    def test_get_agent_installers_trash(self):
        self.gmp.get_agent_installers(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_agent_installers trash="1"/>'
        )

    def test_get_agent_installers_with_filter_string(self):
        self.gmp.get_agent_installers(
            filter_string="name=foo",
        )

        self.connection.send.has_been_called_with(
            b'<get_agent_installers filter="name=foo"/>'
        )

    def test_get_agent_installers_with_filter_id(self):
        self.gmp.get_agent_installers(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_agent_installers filt_id="f1"/>'
        )

    def test_get_agent_installers_with_details(self):
        self.gmp.get_agent_installers(details=True)

        self.connection.send.has_been_called_with(
            b'<get_agent_installers details="1"/>'
        )

    def test_get_agent_installers_without_details(self):
        self.gmp.get_agent_installers(details=False)

        self.connection.send.has_been_called_with(
            b'<get_agent_installers details="0"/>'
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyCredentialStoreTestMixin:
    def test_modify_credential_store(self):
        self.gmp.modify_credential_store(credential_store_id="cs1")

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences/>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential_store(credential_store_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential_store(credential_store_id="")

    def test_modify_credential_store_with_active(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            active=True,
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<active>1</active>"
            b"<preferences/>"
            b"</modify_credential_store>"
        )

        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            active=False,
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<active>0</active>"
            b"<preferences/>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_host(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            host="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<host>foo</host>"
            b"<preferences/>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_port(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            port=1234,
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<port>1234</port>"
            b"<preferences/>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_path(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            path="/foo/bar",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<path>/foo/bar</path>"
            b"<preferences/>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_comment(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            comment="ahoi",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<comment>ahoi</comment>"
            b"<preferences/>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_app_id(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            app_id="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences>"
            b"<app_id>foo</app_id>"
            b"</preferences>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_client_cert(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            client_cert="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences>"
            b"<client_cert>Zm9v</client_cert>"
            b"</preferences>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_client_key(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            client_key="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences>"
            b"<client_key>Zm9v</client_key>"
            b"</preferences>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_client_pkcs12_file(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            client_pkcs12_file="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences>"
            b"<client_pkcs12_file>Zm9v</client_pkcs12_file>"
            b"</preferences>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_passphrase(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            passphrase="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences>"
            b"<passphrase>foo</passphrase>"
            b"</preferences>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_server_ca_cert(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            server_ca_cert="foo",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<preferences>"
            b"<server_ca_cert>Zm9v</server_ca_cert>"
            b"</preferences>"
            b"</modify_credential_store>"
        )

    def test_modify_credential_store_with_all(self):
        self.gmp.modify_credential_store(
            credential_store_id="cs1",
            active=False,
            host="localhost",
            port="80",
            path="/api",
            comment="why was 6 afraid of 7? because 7 8 9",
            app_id="appId",
            client_cert="clientCert",
            client_key="clientKey",
            client_pkcs12_file="clientPkcs12File",
            passphrase="secret",
            server_ca_cert="serverCaCert",
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential_store credential_store_id="cs1">'
            b"<active>0</active>"
            b"<host>localhost</host>"
            b"<port>80</port>"
            b"<path>/api</path>"
            b"<comment>why was 6 afraid of 7? because 7 8 9</comment>"
            b"<preferences>"
            b"<app_id>appId</app_id>"
            b"<client_cert>Y2xpZW50Q2VydA==</client_cert>"
            b"<client_key>Y2xpZW50S2V5</client_key>"
            b"<client_pkcs12_file>Y2xpZW50UGtjczEyRmlsZQ==</client_pkcs12_file>"
            b"<passphrase>secret</passphrase>"
            b"<server_ca_cert>c2VydmVyQ2FDZXJ0</server_ca_cert>"
            b"</preferences>"
            b"</modify_credential_store>"
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_report_format import GmpCloneReportFormatTestMixin
from .test_delete_report_format import GmpDeleteReportFormatTestMixin
from .test_get_report_format import GmpGetReportFormatTestMixin
from .test_get_report_formats import GmpGetReportFormatsTestMixin
from .test_import_report_format import GmpImportReportFormatTestMixin
from .test_modify_report_format import GmpModifyReportFormatTestMixin
from .test_verify_report_format import GmpVerifyReportFormatTestMixin

__all__ = (
    "GmpCloneReportFormatTestMixin",
    "GmpDeleteReportFormatTestMixin",
    "GmpGetReportFormatTestMixin",
    "GmpGetReportFormatsTestMixin",
    "GmpImportReportFormatTestMixin",
    "GmpModifyReportFormatTestMixin",
    "GmpVerifyReportFormatTestMixin",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand, XmlError

from .._entity_id import EntityID
from ._report_formats import ReportFormatType


class Reports:
    @classmethod
    def delete_report(cls, report_id: EntityID) -> Request:
        """Deletes an existing report

        Args:
            report_id: UUID of the report to be deleted.
        """
        if not report_id:
            raise RequiredArgument(
                function=cls.delete_report.__name__, argument="report_id"
            )

        cmd = XmlCommand("delete_report")
        cmd.set_attribute("report_id", str(report_id))

        return cmd

    @classmethod
    def get_report(
        cls,
        report_id: EntityID,
        *,
        filter_string: str | None = None,
        filter_id: str | None = None,
        delta_report_id: EntityID | None = None,
        report_format_id: str | ReportFormatType | None = None,
        ignore_pagination: bool | None = None,
        details: bool | None = True,
    ) -> Request:
        """Request a single report

        Args:
            report_id: UUID of an existing report
            filter_string: Filter term to use to filter results in the report
            filter_id: UUID of filter to use to filter results in the report
            delta_report_id: UUID of an existing report to compare report to.
            report_format_id: UUID of report format to use
                              or ReportFormatType (enum)
            ignore_pagination: Whether to ignore the filter terms "first" and
                "rows".
            details: Request additional report information details
                     defaults to True
        """
        cmd = XmlCommand("get_reports")

        if not report_id:
            raise RequiredArgument(
                function=cls.get_report.__name__, argument="report_id"
            )

        cmd.set_attribute("report_id", str(report_id))

        cmd.add_filter(filter_string, filter_id)

        if delta_report_id:
            cmd.set_attribute("delta_report_id", str(delta_report_id))

        if report_format_id:
            cmd.set_attribute("format_id", str(report_format_id))

        if ignore_pagination is not None:
            cmd.set_attribute("ignore_pagination", to_bool(ignore_pagination))

        cmd.set_attribute("details", to_bool(details))

        return cmd

    @staticmethod
    def get_reports(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        note_details: bool | None = None,
        override_details: bool | None = None,
        ignore_pagination: bool | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of reports

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            note_details: If notes are included, whether to include note details
            override_details: If overrides are included, whether to include
                override details
            ignore_pagination: Whether to ignore the filter terms "first" and
                "rows".
            details: Whether to exclude results
        """
        cmd = XmlCommand("get_reports")

        if filter_string:
            cmd.set_attribute("report_filter", filter_string)

        if filter_id:
            cmd.set_attribute("report_filt_id", str(filter_id))

        if note_details is not None:
            cmd.set_attribute("note_details", to_bool(note_details))

        if override_details is not None:
            cmd.set_attribute("override_details", to_bool(override_details))

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if ignore_pagination is not None:
            cmd.set_attribute("ignore_pagination", to_bool(ignore_pagination))

        return cmd

    @classmethod
    def import_report(
        cls,
        report: str,
        task_id: EntityID,
        *,
        in_assets: bool | None = None,
    ) -> Request:
        """Import a Report from XML

        Args:
            report: Report XML as string to import. This XML must contain
                a :code:`<report>` root element.
            task_id: UUID of task to import report to
            in_asset: Whether to create or update assets using the report
        """
        if not report:
            raise RequiredArgument(
                function=cls.import_report.__name__, argument="report"
            )

        cmd = XmlCommand("create_report")

        if not task_id:
            raise RequiredArgument(
                function=cls.import_report.__name__, argument="task_id"
            )

        cmd.add_element("task", attrs={"id": str(task_id)})

        if in_assets is not None:
            cmd.add_element("in_assets", to_bool(in_assets))

        try:
            cmd.append_xml_str(report)
        except XmlError as e:
            raise InvalidArgument(
                f"Invalid xml passed as report to import_report {e}"
            ) from None

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Package for supported Greenbone Protocol versions.

Currently GMP version 22.4, `GMP version 22.5`_,
`GMP version 22.6`_, `GMP version 22.7`_
and `OSP version 1`_ are supported.

.. _GMP version 22.5:
    https://docs.greenbone.net/API/GMP/gmp-22.5.html
.. _GMP version 22.6:
    https://docs.greenbone.net/API/GMP/gmp-22.6.html
.. _GMP version 22.7:
    https://docs.greenbone.net/API/GMP/gmp-22.7.html
.. _OSP version 1:
    https://docs.greenbone.net/API/OSP/osp-1.2.html
"""



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.protocols.core import Request


class RequestMock:
    def __init__(self, data: bytes) -> None:
        self._data = data

    def __bytes__(self) -> bytes:
        return self._data


class RequestTestCase(unittest.TestCase):
    def test_request(self) -> None:
        # request is just a protocol and can be implemented by several classes
        request = RequestMock(b"some data")
        self.assertIsInstance(request, Request)

        self.assertEqual(bytes(request), b"some data")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .alerts import (
    GmpCloneAlertTestMixin,
    GmpCreateAlertTestMixin,
    GmpDeleteAlertTestMixin,
    GmpGetAlertsTestMixin,
    GmpGetAlertTestMixin,
    GmpModifyAlertTestMixin,
    GmpTestAlertTestMixin,
    GmpTriggerAlertTestMixin,
)


class Gmpv224CloneAlertTestCase(GmpCloneAlertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateAlertTestCase(GmpCreateAlertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeleteAlertTestCase(GmpDeleteAlertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetAlertTestCase(GmpGetAlertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetAlertsTestCase(GmpGetAlertsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyAlertTestCase(GmpModifyAlertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224TestAlertTestCase(GmpTestAlertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224TriggerAlertTestCase(GmpTriggerAlertTestMixin, Gmpv224TestCase):
    pass
